//=====================================================================================
//               Developed As An Addition To Matt Pietrek's LibCTiny.lib
//                            By Fred Harris, April 2016
//
//                   cl fgets.cpp /GS- /c /W3 /DWIN32_LEAN_AND_MEAN 
//=====================================================================================
#include <windows.h>
#include "stdlib.h"
#include "stdio.h"

char* __cdecl fgets(char* pBuffer, int iSize, FILE* fp)   // The only way I know of to implement fgets()
{                                                         // using the WinApi is to use ReadFile() with 
 bool blnAddCharToBuffer=true;                            // the 3rd parameter...
 DWORD lpBytesRead=0;                                     //
 int iCtr=0;                                              // DWORD nNumberOfBytesToRead
 char c=0;                                                //
                                                          // set to 1 so as to indicate a read of 1 byte.
 if(fp!=INVALID_HANDLE_VALUE)                             // The 4th parameter...
 {                                                        //
    pBuffer[0]=0;                                         // LPDWORD lpNumberOfBytesRead 
    do                                                    // 
    {                                                     // ...will equal zero if the end of the file is
       ReadFile((HANDLE)fp,&c,1,&lpBytesRead,NULL);       // reached.  In that case (see left) a LF is 
       if(lpBytesRead==0)   // hit eof                    // placed in the buffer pBuffer, and a NULL byte
       {                                                  // stored after that.  So the basic plan of 
          if(iCtr)                                        // operation is to keep reading through the file
          {                                               // byte by byte testing just what is being read
             pBuffer[iCtr]=10;                            // in char c and of course checking for the end
             pBuffer[iCtr+1]=0;                           // of the file as indicated by an lpBytesRead
          }                                               // equal zero.  If an ordinary character is being
          if(pBuffer[0])                                  // read it gets stored in pBuffer at iCtr.  A
             return pBuffer;                              // CR (Carriage Return - 0x0D) indicates the end
          else                                            // of the text line being read, and a LF (Line
             return NULL;                                 // Feed - 0x0A) is written to that position (that
       }                                                  // is what CRT fgets() does).  That is followed by
       if(iCtr==iSize-1)  // user doesn't want anymore    // a NULL byte.  Where this all gets tricky is 
       {                                                  // if the user specifies an iSize parameter in
          blnAddCharToBuffer=false;                       // the fgets() function call that is less than the
          pBuffer[iCtr+0] = 10;                           // length of the line as determined by the CR
          pBuffer[iCtr+1] = 0;                            // symbol.  By the way, if Notepad is used to
       }                                                  // create the text file, each line will be
       if(c==13)   // hit end of line                     // delimited by CrLf pairs, but fgets just term-
       {                                                  // inates returned strings with LF chars.  
          if(blnAddCharToBuffer)                          // Anyway, the code above left tests iCtr against
          {  // user wants whlole line                    // iSize, and chars have to be prevented from
             pBuffer[iCtr]=10;                            // being written to the pBuffer when iSize is
             iCtr++;                                      // approached.  That's what the blnAddCharsToBuffer
             pBuffer[iCtr]=0;                             // bool is for.  Because you see, the do loop must
             blnAddCharToBuffer=false;                    // continue reading characters from the FILE* 
          }                                               // buffer even after iSize is reached, because if
       }                                                  // it doesn't, the next call to gets() for the next
       if(blnAddCharToBuffer)                             // line will start at the last FILE* position.  I
          pBuffer[iCtr]=c;                                // realize this is tricky to understand.  Think
       if(c==10)   // return                              // about it!  After reading a 0x0A (LF - 10 dec),
          break;                                          // the do loop is exited with a break, and the 
       iCtr++;                                            // function returns pBuffer.  Note that the function
                                                          // returns NULL if end of file is hit.
    }while(true);
 }

 return pBuffer;
}


wchar_t* __cdecl fgetws(wchar_t* pBuffer, int iSize, FILE* fp)  // This code assumes the file being read
{                                                               // is an ansi file, so it calls
 char* pAsciBuffer=NULL;                                        // MultiByteToWideChar() so as to return
 char* pRet=NULL;                                               // the desired wide character string.
 
 printf("Called fgetws()\n");
 memset(pBuffer,0,iSize*2);
 pAsciBuffer=(char*)malloc(iSize);
 if(!pAsciBuffer)
    return NULL;
 memset(pAsciBuffer,0,iSize);
 pRet=fgets(pAsciBuffer,iSize,fp);
 printf("%s",pAsciBuffer);
 if(pRet)
    MultiByteToWideChar(CP_ACP,0,pAsciBuffer,(int)strlen(pAsciBuffer),pBuffer,iSize);
 free(pAsciBuffer);
 if(pRet)
    return pBuffer;
 else
    return NULL; 
}


void __cdecl rewind(FILE* fp)
{
 SetFilePointer((HANDLE)fp,0,NULL,FILE_BEGIN);
}


size_t __cdecl Lof(FILE* fp)
{
 DWORD lpNumberOfBytesRead=0;
 size_t iReadSomething=0;
 size_t iCtr=0;
 char c=0;

 if(fp!=INVALID_HANDLE_VALUE)
 {
    do
    {
       ReadFile((HANDLE)fp,&c,1,&lpNumberOfBytesRead,NULL);
       if(lpNumberOfBytesRead)
       {
          if(c==10)
          {  iReadSomething=0;
             iCtr++;
          }
          else
             iReadSomething=1;
       }
       else
          break;
    }while(true);
 }
 rewind(fp);

 return iCtr+iReadSomething; 
}

/*
Retrieving a File Pointer

The SetFilePointer function can be used to query the current file pointer position, by specifying a move method 
of FILE_CURRENT and a distance of zero.

HANDLE hFile; 
 
DWORD dwCurrentFilePosition; 
 
dwCurrentFilePosition = SetFilePointer( 
    hFile, // must have GENERIC_READ and/or GENERIC_WRITE 
    0,     // do not move pointer 
    NULL,  // hFile is not large enough to need this pointer 
    FILE_CURRENT);  // provides offset from current position 
 
The following macro can be used to obtain the current file pointer position for files smaller than 2 gigabytes in size:

#define GetFilePointer(hFile) SetFilePointer(hFile, 0, NULL, FILE_CURRENT)
 

*/
